#!/usr/bin/python3
# kate: space-indent on; indent-width 4; replace-tabs on; indent-mode python; remove-trailing-space modified;
# vim: expandtab ts=4

############################################################################
#   Copyright © 2015 Jonathan Riddell                                      #
#   Copyright © 2015-2016 José Manuel Santamaría Lema <panfaust@gmail.com> #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
############################################################################

# script to bump build-dep versions in KDE packaging
# run in directory with debian/control file

#Handle Crtl+C
import signal
from libka.ka_print import *
def signal_handler(signal, frame):
        ka_print_plain('Aborted by user request')
        sys.exit(130) #http://www.tldp.org/LDP/abs/html/exitcodes.html
signal.signal(signal.SIGINT, signal_handler)

import argparse
import os
import sys
import re
import subprocess
import json
import warnings

from libka.control_edit import *
from libka.ka_data_utils import *


def bump_build_depends(releasetype, dist):
    #Open the build depends map
    build_depends_map_file = "dev-package-name-lists/%s-%s.json" % (releasetype, dist)
    try:
        build_depends_map = readJsonDataFile(build_depends_map_file,catch_exceptions=False)
    except FileNotFoundError:
        print("Build depends map file path:")
        print(getDataFilesPath()+build_depends_map_file)
        print("Build depends map file not found, aborting.")
        print("You can update the build depends map file with ka-update-metadata -d <distribution> -r <releasetype>")
        sys.exit(1)

    #Open the overrides file if exists
    overrides_file = "dev-package-name-lists/%s-%s-overrides.json" % (releasetype, dist)
    try:
        overrides_map = readJsonDataFile(overrides_file,catch_exceptions=False)
    except FileNotFoundError:
        overrides_map = {}

    #Add the overrides to the build depends map
    build_depends_map.update(overrides_map)

    #Sanitize the control file
    sanitize_control_file('./debian/control')

    #Open the control file
    src_pkg, bin_pkg_list, bin_pkg_map = parse_control('./debian/control');

    warnings.simplefilter('ignore', UserWarning) #Ignore the warnings from python-debian
    warnings.simplefilter('ignore', FutureWarning) #Ignore the warnings from python-debian

    #Bump the build depends using the map from the json file
    bump_version_with_map(src_pkg, 'Build-Depends', build_depends_map)
    if 'Build-Depends-Indep' in src_pkg:
	    bump_version_with_map(src_pkg, 'Build-Depends-Indep', build_depends_map)

    #Bump the depends of the -dev packages as well
    for bin_pkg_name in bin_pkg_list:
        if bin_pkg_name in build_depends_map:
            bin_pkg_to_change = bin_pkg_map[bin_pkg_name]
            if 'Depends' in bin_pkg_to_change:
                bump_version_with_map(bin_pkg_to_change, 'Depends', build_depends_map)

    warnings.simplefilter('default', FutureWarning) #Reset the future warnings
    warnings.simplefilter('default', UserWarning) #Reset the user warnings

    #Dump the contents of the control file
    dump_control('./debian/control',src_pkg, bin_pkg_list, bin_pkg_map)

def bump_autopkgtest_depends(releasetype, dist):

    #Before anything try to sanitize the debian/tests/control file, if not found, abort
    try:
        sanitize_control_file("debian/tests/control")
    except FileNotFoundError:
        return

    #Read the binary package version map generated with ka-update-metadata
    bin_pkg_version_map_file = "bin-package-name-lists/%s-%s.json" % (releasetype, dist)
    try:
        bin_pkg_version_map = readJsonDataFile(bin_pkg_version_map_file,catch_exceptions=False)
    except FileNotFoundError:
        ka_print_error("Binary package version map file path:")
        ka_print_error( os.path.join(getDataFilesPath(), bin_pkg_version_map_file) )
        ka_print_error("Binary package version map file not found, aborting.")
        ka_print_error("You can update the binary package version map file with:")
        ka_print_error("ka-update-metadata -d <distribution> -r <releasetype>")
        sys.exit(1)

    #Open the overrides file if exists
    overrides_file = "bin-package-name-lists/%s-%s-overrides.json" % (releasetype, dist)
    try:
        overrides_map = readJsonDataFile(overrides_file,catch_exceptions=False)
    except FileNotFoundError:
        overrides_map = {}

    #Add the overrides to the binary package version map
    bin_pkg_version_map.update(overrides_map)

    #Iterate over tests(paragraphs) and bump the Depends for each one
    warnings.simplefilter('ignore', UserWarning) #Ignore the warnings from python-debian
    tests_control = deb822.Packages.iter_paragraphs(open('debian/tests/control','r'))
    new_paragraphs = []
    for paragraph in tests_control:
        bump_version_with_map(paragraph, 'Depends', bin_pkg_version_map)
        new_paragraphs.append(paragraph)
    warnings.simplefilter('default', UserWarning) #Reset the user warnings

    #Dump the contents of the new paragraphs to the file
    new_control_file = open('debian/tests/control', 'w')
    for paragraph in new_paragraphs:
        new_control_file.write(paragraph.dump())
        new_control_file.write('\n')
    new_control_file.close()

if __name__ == "__main__":
    #Find out the default distribution
    ka_config = getConfigParser()
    default_dist = ka_config['global']['ubuntu-unstable-name']

    parser = argparse.ArgumentParser(description="This script must be executed inside a package directory "
        "with debian/* files. It bumps all the build dependencies of frameworks/plasma/applications.")
    parser.add_argument("-d", "--dist", required=False, default=default_dist, help="Distribution name")
    parser.add_argument("-r", "--releasetype", required=True, help="Type [frameworks,plasma,applications]")
    parser.add_argument("-t", "--autopkgtests", action='store_true', required=False, default=False,
        help="Bump also the depends of debian/tests/control")

    args = parser.parse_args()

    bump_build_depends(args.releasetype, args.dist)
    if args.autopkgtests:
        bump_autopkgtest_depends(args.releasetype, args.dist)

    #Print the git diff
    print("=== bump-build-dep-versions diff start")
    subprocess.check_call(["git", "--no-pager", "diff"])
    print("=== bump-build-dep-versions diff end")
