#!/usr/bin/python3
# -*- coding: utf-8 -*-
# kate: space-indent on; indent-width 4; replace-tabs on; indent-mode python; remove-trailing-space modified;
# vim: expandtab ts=4

############################################################################
#   Copyright © 2015-2016 José Manuel Santamaría Lema <panfaust@gmail.com> #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
############################################################################

import os
import json
import subprocess
import codecs

from debian import deb822
from debian.changelog import Changelog, Version

from libka.ka_configuration import *
from libka.package_info import *
from libka.ka_data_utils import *

#Downloads all the tarballs from depot for the specified release type
#Returns True if the tarball[s] were sucessfully downloaded

def download_tarballs_from_depot(release_type,tarball='*'):

    #Read configuration files
    ka_config = getConfigParser()
    config_locations = ka_config['tarball-locations']
    rt_versions = readJsonDataFile("versions.json")

    #Prepare the destination directory
    if release_type != "kde-l10n":
        version = rt_versions[release_type]
    else:
        version = rt_versions["applications"]
    dest_dir = config_locations[release_type] + "/" + version
    dest_dir = os.path.expanduser(dest_dir)
    os.makedirs(dest_dir, exist_ok=True)

    #Find out the stability
    version_parts = version.split(".")
    last_digit = int(version_parts[-1])
    if last_digit >= 80:
        stability = "unstable"
    else:
        stability = "stable"

    #Find out which subdirectories we have to inspect in the ftp
    ftp_subdirs = []
    if release_type == "frameworks":
        ftp_subdirs = ["","portingAids/"]
    elif release_type == "plasma":
        ftp_subdirs = [""]
    elif release_type == "applications":
        ftp_subdirs = ["src/"]
    elif release_type == "kde-l10n":
        release_type = "applications"
        ftp_subdirs = ["src/kde-l10n/"]

    url_template = "racnoss.kde.org:/srv/archives/ftp/%s/%s/%s/%s%s-[0-9.]*.tar.xz"

    tarball_downloaded = False
    for subdir in ftp_subdirs:
        url = url_template % (stability,release_type,version,subdir,tarball)
        print(url)
        #os.system("rsync -avP %s %s" % (url,dest_dir))
        try:
            download_command = ["rsync","-av","--progress",url,dest_dir]
            print(" ".join(download_command))
            subprocess.check_call(download_command)
            tarball_downloaded = True
            break
        except:
            pass
    return tarball_downloaded


#Downloads the tarball of the current version but it doesn't symlink it; see
#the other function below

def download_tarball_uscan(releaseType):
    #Read configuration files
    ka_config = getConfigParser()
    config_locations = ka_config['tarball-locations']
    #Create destination directory
    if releaseType == "" or releaseType == "other":
        destdir = ka_config['areas']['build-area']
    else:
        destdir = ka_config['tarball-locations'][releaseType]
        version = readJsonDataFile("versions.json")[releaseType]
        destdir += "/" + version
        destdir = os.path.expanduser(destdir)
    os.makedirs(destdir, exist_ok=True)
    #Download tarball to the destination directory.
    program = ["uscan", "--download-current-version", "--no-symlink", "--destdir="+destdir]
    if releaseType == "" or releaseType == "other":
        program += ["--rename"]
    subprocess.call(program)


#Creates a link to the upstream tarball, if the target file pointed by the link
#doesn't exit, the tarball is downloaded before making the link

def link_upstream_tarball(releaseType):

    #Read configuration files
    ka_config = getConfigParser()
    config_locations = ka_config['tarball-locations']

    #Find out the upstream name
    try:
        upstream_name = upstreamNameFromWatch()
    except Exception as e:
        print("WARNING: Failed to download tarball.")
        print(e)
        return
    
    #Find out package info - src package name, upstream version
    changelog = Changelog()
    changelog.parse_changelog(codecs.open('./debian/changelog', 'r', 'utf-8'))
    src_package_name = changelog.get_package()
    upstream_version = changelog.get_version().upstream_version
    #Handle packages with updated tarballs
    #e.g. 5.0.0a version instead of 5.0.0
    last_char = upstream_version[-1]
    if last_char.isalpha():
        real_upstream_version = upstream_version.rstrip(last_char)
    else:
        real_upstream_version = upstream_version

    #Find out the link name    
    build_area_dir = ka_config['areas']['build-area']
    try:
        os.makedirs(build_area_dir)
    except:
        pass
    link_name = build_area_dir + '/' + src_package_name + '_' + upstream_version + '.orig.tar.xz'

    #If the release type is unknown just download the tarball with uscan
    #if it's not there anymore.
    if releaseType == "" or releaseType == "other":
        if not os.path.exists(link_name):
            download_tarball_uscan("")
        return

    #Find out the tarball link target path
    rt_versions = readJsonDataFile("versions.json")
    link_target = os.path.expanduser(ka_config['tarball-locations'][releaseType])
    link_target += '/' + rt_versions[releaseType] + '/'
    link_target += upstream_name + '-' + real_upstream_version + ".tar.xz"

    #If the tarball isn't there already, download it with uscan
    if not os.path.isfile(link_target):
        if not download_tarballs_from_depot(releaseType,upstream_name):
            download_tarball_uscan(releaseType)

    #Create the link
    try:
        os.symlink(link_target,link_name)
    except:
        pass


# Returns the orig tarball path in build area

def get_orig_tarball_path():

# FIXME: This is just a partial C&P of link_upstream_tarball()
#        we could avoid this code duplication creating a class like this
#   class OrigTarball:
#       def __init__(self):
#           # Code to initialize:
#           # self._upstream_name
#           # self._src_package_name
#           # self._upstream_version
#       def get_orig_tarball_path(self):
#           # Do the same than the current get_orig_tarball_path()
#       def link_upstream_tarball(self):
#           # Do the same than the current link_upstream_tarball but
#           # find out release type with getReleaseType() from libka.utils

    #Read configuration files
    ka_config = getConfigParser()
    config_locations = ka_config['tarball-locations']

    #Find out the upstream name
    try:
        upstream_name = upstreamNameFromWatch()
    except:
        print("WARNING: Failed to download tarball.")
        return
    
    #Find out package info - src package name, upstream version
    changelog = Changelog()
    changelog.parse_changelog(open('./debian/changelog', 'r'))
    src_package_name = changelog.get_package()
    upstream_version = changelog.get_version().upstream_version
    #Handle packages with updated tarballs
    #e.g. 5.0.0a version instead of 5.0.0
    last_char = upstream_version[-1]
    if last_char.isalpha():
        real_upstream_version = upstream_version.rstrip(last_char)
    else:
        real_upstream_version = upstream_version

    #Find out the link name
    build_area_dir = ka_config['areas']['build-area']
    try:
        os.makedirs(build_area_dir)
    except:
        pass
    link_name = build_area_dir + '/' + src_package_name + '_' + upstream_version + '.orig.tar.xz'
    return link_name


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Link the orig tarball.")
    parser.add_argument("-r", "--releasetype",
        help="KDE Release Type [frameworks,plasma,applications]",
        required=True)
    args = parser.parse_args()
    link_upstream_tarball(args.releasetype)
