# -*- coding: utf-8 -*-
# kate: space-indent on; indent-width 4; replace-tabs on; indent-mode python; remove-trailing-space modified;
# vim: expandtab ts=4

############################################################################
#   Copyright © 2017 José Manuel Santamaría Lema <panfaust@gmail.com>      #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
############################################################################

from html.parser import HTMLParser
import re
import urllib3

from libka.ka_print import *

#Class to represent the 'excuses' status for a specific source package
class ExcusesPackageInfo():
    __slots__ = [
        "package_name","old_version","new_version",
        "maintainer","section","age",
        "autopkgtests",
        "no_binaries_text",
        "block_request",
        "depends",
        "status"
    ]

    def __init__(self):
        #Init all the attributes to the empty string
        for slot in self.__slots__:
            if slot == "autopkgtests":
                self.autopkgtests = {}
            elif slot == "depends":
                self.depends = {}
            else:
                setattr(self,slot,"")

    def __str__(self):
        slots_values_str_list = []
        for slot in self.__slots__:
            slot_str = "%s='%s'" % ( slot, getattr(self,slot) )
            slots_values_str_list.append(slot_str)
        slots_values_str = ",\n\t".join(slots_values_str_list)
        result = "PackageInfo(%s)" % slots_values_str
        return result

    def print(self):
        #Print package name and versions
        ka_print_plain("Package: %s (from %s to %s)"
            % (self.package_name,self.old_version,self.new_version) )
        #Print maintainer
        ka_print_plain("    Maintainer: %s" % self.maintainer)
        #Print section
        if self.section:
            ka_print_plain("    Section: %s" % self.section)
        #Pint age
        ka_print_plain("    %s days old" % self.age)
        #Print autopkgtests
        for autopkgtest_pkg in sorted(self.autopkgtests.keys()):
            ka_print_plain("    autopkgtest for %s: " % autopkgtest_pkg,end='')
            first_arch = True
            for arch in sorted(self.autopkgtests[autopkgtest_pkg].keys()):
                if not first_arch:
                    ka_print_plain(', ',end='')
                ka_print_plain(arch + ": ",end='')
                autopkgtest_result = self.autopkgtests[autopkgtest_pkg][arch]
                if autopkgtest_result == "Pass":
                    ka_print_good_stuff(autopkgtest_result,end='')
                elif autopkgtest_result == "Regression":
                    ka_print_error(autopkgtest_result,end='')
                elif ( (autopkgtest_result == "Always failed")
                or (autopkgtest_result == "Ignored failure") ):
                    ka_print_warning(autopkgtest_result,end='')
                elif autopkgtest_result.startswith('Test in progress'):
                    ka_print_info(autopkgtest_result,end='')
                else:
                    ka_print_plain(autopkgtest_result,end='')
                first_arch = False
            ka_print_plain('') #end with a newline
        #Print no binaries text (if any)
        if self.no_binaries_text:
            ka_print_error("    %s" % self.no_binaries_text)
        #Print block request text (if any)
        if self.block_request:
            ka_print_plain("    %s" % self.block_request)
        #Print depends
        if self.depends:
            depends_str_list = []
            for depend in sorted(self.depends.keys()):
                depend_str = depend + self.depends[depend]
                depends_str_list.append(depend_str)
            depends_string = ", ".join(depends_str_list)
            ka_print_plain("    Depends: %s" % depends_string)
        #Print status
        if self.status == "Valid candidate":
            ka_print_good_stuff("    %s" % self.status)
        else:
            ka_print_warning("    %s" % self.status)


# Parses an 'update-excuses' webpage
class ExcusesHTMLParser(HTMLParser):
    def __init__(self):
        super(ExcusesHTMLParser,self).__init__()
        self._body_found = False
        self._first_ul_found = False
        self._ul_found = False
        self._nested_ul_found = False
        self._li_found = False
        self._pkg_header_data = ""
        self._processing_pkg_header_data = False
        self._current_package_info = ExcusesPackageInfo()
        self._current_package_header_count = 0
        self._current_package_contents_count = 0
        self._processing_autopkgtest_arch = False
        self._processing_autopkgtest_result = False
        self._current_autopkgtest_package = None
        self._processing_depends = False
        self._current_depend = None
        self._package_info_dict = {}
        self._debug_mode = False

    def set_debug_mode(self):
        self._debug_mode = True

    def parse_excuses_url(self,excuses_url):
        #Get the update excuses web page
        http = urllib3.PoolManager()
        response = http.request("GET",excuses_url)
        html_str = response.data.decode('utf-8')
        self.feed(html_str)

    def handle_starttag(self, tag, attrs):
        #Ignore everything until we find the body
        if tag != "body":
            if not self._body_found:
                return
        else:
            self._body_found = True
        #Ignore everything until we find the first <ul>
        #(which would have the info for the first package)
        if tag != "ul":
            if not self._first_ul_found:
                return
        else:
            self._first_ul_found = True
        #Check ul's and li's
        if tag == "ul":
            if not self._ul_found:
                self._ul_found = True
            else:
                self._nested_ul_found = True
        elif tag == "li":
            self._li_found = True

    def handle_endtag(self, tag):
        #Ignore everything until we find the first package info
        if not (self._body_found and self._first_ul_found):
            return
        #Check ul's and li's
        if tag == "ul":
            if self._nested_ul_found:
                self._nested_ul_found = False
                self._current_package_header_count = 0
                self._current_package_contents_count = 0
                package_name = self._current_package_info.package_name
                self._package_info_dict[package_name] = self._current_package_info
                self._current_package_info = ExcusesPackageInfo()
            else:
                self._ul_found = False
        elif tag == "li":
            self._li_found = False

    def _process_pkg_header_data(self):
        if self._processing_pkg_header_data:
            if self._debug_mode:
                ka_print_plain(self._pkg_header_data)
            re_match = re.search( '(.*) \((.*) to (.*)\)', self._pkg_header_data )
            self._current_package_info.package_name = re_match.group(1).strip()
            self._current_package_info.old_version = re_match.group(2).strip()
            self._current_package_info.new_version = re_match.group(3).strip()
            self._processing_pkg_header_data = False

    def handle_data(self, data):
        #Ignore everything until we find the first package info
        if not (self._body_found and self._first_ul_found):
            return
        if self._ul_found:
            if not self._nested_ul_found:
                if self._li_found:
                    #This provides package_name, old_version and new_version
                    self._current_package_header_count += 1
                    data = data.strip()
                    if self._debug_mode:
                        ka_print_plain("Encountered data         %s: %s"
                            % (self._current_package_header_count,data))
                    if not self._processing_pkg_header_data:
                        self._pkg_header_data = ""
                    self._processing_pkg_header_data = True
                    self._pkg_header_data += " " + data
            else:
                self._process_pkg_header_data()
                if self._li_found:
                    self._current_package_contents_count += 1
                    data = data.strip()
                    if self._debug_mode:
                        ka_print_plain("Encountered nested data  %s: %s"
                            % (self._current_package_contents_count,data))
                    if data.startswith('Maintainer: '):
                        self._current_package_info.maintainer = data.split('Maintainer: ')[1]
                    elif data.startswith('Section: '):
                        self._current_package_info.section = data.split('Section: ')[1]
                    elif data.endswith(' days old'):
                        self._current_package_info.age = data.split(' days old')[0]
                    elif "no binaries" in data:
                        self._current_package_info.no_binaries_text = data
                    elif data.startswith('Not considered'):
                        self._current_package_info.status = 'Not considered'
                        self._processing_autopkgtest_arch = False
                        self._processing_autopkgtest_result = False
                        self._processing_depends = False
                    elif data.startswith('Valid candidate'):
                        self._current_package_info.status = 'Valid candidate'
                        self._processing_autopkgtest_arch = False
                        self._processing_autopkgtest_result = False
                        self._processing_depends = False
                    elif data.startswith('autopkgtest for '):
                        self._current_autopkgtest_package = data.split('autopkgtest for ')[1]
                        self._current_autopkgtest_package = self._current_autopkgtest_package.strip(': ')
                        self._current_package_info.autopkgtests[self._current_autopkgtest_package] = {}
                        self._processing_autopkgtest_arch = True
                        self._processing_autopkgtest_result = False
                    elif self._processing_autopkgtest_arch:
                        if data.islower():
                            self._current_autopkgtest_arch = data
                            self._processing_autopkgtest_arch = False
                            self._processing_autopkgtest_result = True
                    elif self._processing_autopkgtest_result:
                        try:
                            first_char = data[0]
                        except:
                            first_char = ''
                        if first_char.isupper():
                            arch = self._current_autopkgtest_arch
                            package = self._current_package_info.package_name
                            autopkgtest_package = self._current_autopkgtest_package
                            self._current_package_info.autopkgtests[autopkgtest_package][arch] = data
                            self._processing_autopkgtest_arch = True
                            self._processing_autopkgtest_result = False
                    elif "block request" in data:
                        self._current_package_info.block_request = data
                    elif data.startswith('Depends: '):
                        self._processing_depends = True
                    elif self._processing_depends:
                        if not data.startswith('('):
                            self._current_package_info.depends[data] = ""
                            self._current_depend = data
                        else:
                            self._current_package_info.depends[self._current_depend] = data

        #print("Encountered some data  :", data)

    def get_package_info(self):
        return self._package_info_dict

