#!/usr/bin/python3
# -*- coding: utf-8 -*-
# kate: space-indent on; indent-width 4; replace-tabs on; indent-mode python; remove-trailing-space modified;
# vim: expandtab ts=4

############################################################################
#   Copyright © 2017 José Manuel Santamaría Lema <panfaust@gmail.com>      #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
############################################################################

#Small library to provide directed graphs, if you want to see it in action,
#just execute 'kagraph' demo

import pygraphviz as pgv
import time
import subprocess
import tempfile
import re

from libka.ka_print import *
from libka.grep_excuses import *

#Class to represent a package node status
class KAGraphNodeStatus:
    def __init__(self,name,color,text='',url=''):
        self.name = name
        self.color = color
        self.text = text
        self.url = url


#Generic graph class, if you feed in packages and edges you will be able
#to dump this graph to a dot, ps or pdf file and/or view the graph with
#a pdf viewer
class KAGraph(pgv.AGraph):
    def __init__(self,title=''):
        self._pgv_graph = pgv.AGraph(directed=True,strict=False,name=title)
        self._pgv_graph.graph_attr['rankdir'] = 'LR'
        self._pgv_graph.node_attr['style'] = 'filled'
        self._pgv_graph.add_node('GOD',label='',width='0',fixedsize='true',style='invis')
        self._title = title
        self._pgv_graph.graph_attr['title'] = title
        self._date_string = ''
        self._package_status_list = []
        self._arch_list = None # this means all architectures by default
        self._expected_version = ""
        self._different_versions = []

    def _update_label(self):
        #Show the label on top
        self._pgv_graph.graph_attr['labelloc'] = 't'
        #Set the label string with the title and the timestamp if any
        self._pgv_graph.graph_attr['label'] = (
            '< <FONT FACE="boldfontname" POINT-SIZE="40">%s</FONT> <BR/>'
            '<FONT POINT-SIZE="30">Generated on %s </FONT> >'
            % (self._title, self._date_string) )

    def get_pgv_graph(self):
        return self._pgv_graph

    def set_expected_version(self,version):
        self._expected_version = version

    def set_different_versions(self,diff_version_list):
        self._different_versions = diff_version_list

    def set_arch_list(self, arch_list_str):
        if arch_list_str == 'any':
            self._arch_list = None
        else:
            self._arch_list = arch_list_str.split(',')

    def add_package(self,package_name):
        self._pgv_graph.add_node(package_name)
        self._pgv_graph.add_edge('GOD',package_name,style='invis')

    def add_edge(self,package1,package2):
        self._pgv_graph.add_edge(package1,package2)

    def transitive_reduction(self):
        self._pgv_graph.tred()

    def set_package_status_list(self,status_list):
        self._package_status_list = status_list

    def set_package_status(self,package,status):
        try:
            node = self._pgv_graph.get_node(package)
        except KeyError:
            raise KeyError("Package %s not found in graph" % package)
            return
        node.attr['fillcolor'] = status.color
        node.attr['href'] = status.url

    def set_package_url(self,package,url):
        try:
            node = self._pgv_graph.get_node(package)
        except KeyError:
            raise KeyError("Package %s not found in graph" % package)
            return
        node.attr['href'] = url

    def populate_from_bd_relations_map(self,bd_relations_map):
        for package in sorted(bd_relations_map):
            self.add_package(package)
            build_depends = bd_relations_map[package]
            for build_depend in sorted(build_depends):
                self.add_edge(build_depend,package)

    def highlight_package(self,package,label=''):
        try:
            node = self._pgv_graph.get_node(package)
        except KeyError:
            raise KeyError("Package %s not found in graph" % package)
            return
        #Highlight the package changing its shape
        node.attr['shape'] = 'tripleoctagon'
        node.attr['style'] = 'bold,rounded,filled'
        ##Highglight the package drawing a rectangle box around it
        #s = self._pgv_graph.add_subgraph(package,"cluster_" + package)
        #s.graph_attr['shape'] = 'box'
        #s.graph_attr['style'] = 'bold,rounded'
        #s.graph_attr['label'] = label

    def add_legend(self):
        #Create the cluster with the proper label and shape
        s = self._pgv_graph.add_subgraph('','cluster_legend')
        s.graph_attr['label'] = 'Legend'
        s.graph_attr['style'] = 'bold,rounded'
        #Add a node for each possible status
        for status in self._package_status_list:
            if status.text:
                node_name = status.text
            else:
                node_name = status.name
            s.add_node(node_name)
            node = s.get_node(node_name)
            node.attr['fillcolor'] = status.color
            node.attr['shape'] = 'plaintext'
            self._pgv_graph.add_edge(node,'GOD',style='invis')

    def update_timestamp(self):
        self._date_string = time.strftime('%a, %d %b %Y %X %z')
        self._update_label()

    def dump_graph_to_dot(self,file_name):
        self._pgv_graph.write(file_name)

    def dump_graph_to_ps(self,file_name):
        self._pgv_graph.draw(file_name,prog='dot',format='ps2')

    def dump_graph_to_pdf(self,file_name):
        #Dump graph to a temporary PostScript 2 file
        tmp_ps_file = tempfile.NamedTemporaryFile(delete=False)
        self.dump_graph_to_ps(tmp_ps_file.name)
        #Find out the pdf file name to write
        #Convert the ps file to pdf; otherwise URL links won't work, see:
        #http://www.graphviz.org/content/general-comment-how-set-clickable-link-pdf-using-dot-language
        # "Note that for this to work with PDF, you can't use the -Tpdf flag as this
        # relies on Cairo for output, and Cairo PDF does not yet handle links. Instead,
        # use -Tps2. This generates a PostScript file that, when converted to PDF, will
        # support links.
        subprocess.check_call(['ps2pdf',tmp_ps_file.name,file_name])

    def display_graph(self,file_name=None):
        if file_name is None:
            tmp_pdf_file = tempfile.NamedTemporaryFile(delete=False)
            file_name = tmp_pdf_file.name
        self.dump_graph_to_pdf(file_name)
        subprocess.check_call(['okular',file_name])


#Class to represent a build status graph for the official archive;
#in addition to packages(nodes) and edges you must feed in the packages
#status retrieved with launchpadlib
class KAGraphArchiveBuildStatus(KAGraph):

    def __init__(self,dist,title='',):

        #Call the superclass constructor
        super(KAGraphArchiveBuildStatus,self).__init__(title)

        #Define all possible kinds of status
        self._status_ok = KAGraphNodeStatus(name='status_ok',
            color='green3',text="Successfully built")
        self._status_building = KAGraphNodeStatus(name='status_building',
            color='greenyellow',text="Currently building")
        self._status_needsbuilding = KAGraphNodeStatus(name='status_needsbuilding',
            color='yellow',text="Needs building")
        self._status_depwait = KAGraphNodeStatus(name='status_depwait',
            color='lightblue',text="Dependency wait")
        self._status_ftbfs = KAGraphNodeStatus(name='status_ftbfs',
            color='red',text="Failed to build")
        self._status_new = KAGraphNodeStatus(name='status_new',color='chocolate',text="Package in the NEW queue",
            url="https://launchpad.net/ubuntu/%s/+queue" % dist)
        self._status_notfound = KAGraphNodeStatus(name='status_notfound',
            color='grey',text="Package not found")
        self._status_wrongversion = KAGraphNodeStatus(name='status_wrongversion',
            color='firebrick',text="Incorrect version")
        self._status_other = KAGraphNodeStatus(name='status_other',
            color='dimgrey',text="Other")
        self.set_package_status_list([
            self._status_ok, self._status_building, self._status_needsbuilding,
            self._status_depwait, self._status_ftbfs, 
            self._status_new, self._status_wrongversion,
            self._status_notfound, self._status_other
            ])

        #Map to find out which build state is "better"
        self._lp_build_status_level = {
            "Successfully built": 0,
            "Currently building": 3,
            "Uploading build": 3,
            "Needs building": 2,
            "Dependency wait": 1,
            "Failed to build": 5,
            "Chroot problem": 4,
            "Build for superseded Source": 4,
            "Failed to upload": 5,
            "Cancelling build": 3,
            "Cancelled build": 5 }

        #Map to find out the build status from the build status level
        self._build_status_from_level = {
            0:self._status_ok,
            1:self._status_depwait,
            2:self._status_needsbuilding,
            3:self._status_building,
            4:self._status_other,
            5:self._status_ftbfs,
            6:self._status_wrongversion
        }


    #If you set the lpseries parameter as None, the new queue won't be checked, this is interesting
    #for PPA builds
    def set_package_status_from_lp_sources_and_builds(self,all_sources,all_builds,lpseries):
        #FIXME: The following situations are not handled correctly:
        #       - packages with *binaries* waiting in the new queue
        #Write package states in the graph
        for package in all_sources:
            sources = all_sources[package]
            if len(sources) == 0:
                #This package is not on Proposed nor Release
                if lpseries != None:
                    #Check new queue
                    package_uploads_new = lpseries.getPackageUploads(name=package,exact_match=True,status='New')
                    if len(package_uploads_new) == 0:
                        self.set_package_status(package,self._status_notfound)
                    else:
                        self.set_package_status(package,self._status_new)
                else:
                    #Mark the package as not found without checking the new queue
                    self.set_package_status(package,self._status_notfound)
            else:
                #This package is either in proposed or release
                builds = all_builds[package]
                #Check if the package is at the correct version
                if package not in self._different_versions:
                    actual_version = sources[0].source_package_version.split(':')[-1]
                    expected_version = self._expected_version
                    if not actual_version.startswith(expected_version):
                        #This package has a wrong version
                        self.set_package_status(package,self._status_wrongversion)
                        #Set the package node url in the graph
                        url = sources[0].self_link
                        url = re.sub(r"api\.launchpad\.net/[^/]*","launchpad.net",url) + "/+listing-archive-extra"
                        self.set_package_url(package,url)
                        continue
                #Inspect the architecture builds and escalate the build status level up to its higher value
                overall_build_status_level = -1
                for build in builds:
                    #Ignore this build if it's not in the arch list; None means 'consider all'
                    if self._arch_list != None:
                        if build.arch_tag not in self._arch_list:
                            continue
                    #Update status level
                    current_build_status_level = self._lp_build_status_level[build.buildstate]
                    if current_build_status_level > overall_build_status_level:
                        overall_build_status_level = current_build_status_level
                #Set the package node status in the graph
                if overall_build_status_level == -1:
                    #This should not happen, but "shit happens"
                    package_status = self._status_other
                else:
                    package_status = self._build_status_from_level[overall_build_status_level]
                #Check if the package is ok to add url links or check the new queue
                if package_status != self._status_ok:
                    #Set the final package state
                    self.set_package_status(package,package_status)
                    #Set the package node url in the graph
                    url = sources[0].self_link
                    url = re.sub(r"api\.launchpad\.net/[^/]*","launchpad.net",url) + "/+listing-archive-extra"
                    self.set_package_url(package,url)
                else:
                    #Set the final package state
                    self.set_package_status(package,package_status)
                    #Check new queue
                    pass
                    #FIXME: this wouldn't work
                    #if lpseries != None:
                    #    package_uploads_new = lpseries.getPackageUploads(name=package,exact_match=True,status='New')
                    #    if len(package_uploads_new) != 0:
                    #        self.set_package_status(package,self._status_new)



#Class to represent a graph containing a PPA build status, its difference
#compared with KAGraphArchiveBuildStatus is that this class doesn't consider
#the NEW queue (that doesn't make sense for PPAs)
class KAGraphPPABuildStatus(KAGraphArchiveBuildStatus):
    def __init__(self,dist,title=''):
        #Call the superclass constructor
        super(KAGraphPPABuildStatus,self).__init__(dist,title)
        #Delete the new status, we won't check the new queue for ppa builds
        self._package_status_list.remove(self._status_new)

    def set_package_status_from_lp_sources_and_builds(self,all_sources,all_builds):
        #Call the superclass method with lpseries=None
        super(KAGraphPPABuildStatus,self).set_package_status_from_lp_sources_and_builds(
            all_sources,all_builds,None)


#Class to represent a graph with our current proposed migration status, it's
#meant to point out which packages are stuck in -proposed
class KAGraphProposedMigrationStatus(KAGraph):

    def __init__(self,dist,title=''):

        #Call the superclass constructor
        super(KAGraphProposedMigrationStatus,self).__init__(title)

        #Define all possible kinds of status
        self._status_ok = KAGraphNodeStatus(name='status_ok',color='green3',text="already migrated")
        self._status_ftbfs = KAGraphNodeStatus(name='status_ftbfs',color='red',text="FTBFS",
            url="https://people.canonical.com/~ubuntu-archive/proposed-migration/update_excuses.html#\\N")
        self._status_depwait = KAGraphNodeStatus(name='status_depwait',color='lightblue',text="dep wait",
            url="https://people.canonical.com/~ubuntu-archive/proposed-migration/update_excuses.html#\\N")
        self._status_stuck = KAGraphNodeStatus(name='status_stuck',color='orange',text="not migrated yet",
            url="https://people.canonical.com/~ubuntu-archive/proposed-migration/update_excuses.html#\\N")
        self._status_notfound = KAGraphNodeStatus(name='status_notfound',color='grey',text="Package not found")
        self._status_new = KAGraphNodeStatus(name='status_new',color='chocolate',text="Package in the NEW queue",
            url="https://launchpad.net/ubuntu/%s/+queue" % dist)
        self._status_wrongversion = KAGraphNodeStatus(name='status_wrongversion',color='firebrick',
            text="Incorrect version")
        self._status_autopkgtests_running = KAGraphNodeStatus(name='status_autopkgtests_running',
            color='dodgerblue',
            text="Autopkgtests running",
            url="https://people.canonical.com/~ubuntu-archive/proposed-migration/update_excuses.html#\\N")
        self._status_autopkgtests_failed = KAGraphNodeStatus(name='status_autopkgtest_failed',
            color='salmon',
            text="Autopkgtests failed",
            url="https://people.canonical.com/~ubuntu-archive/proposed-migration/update_excuses.html#\\N")
        self._status_valid_candidate = KAGraphNodeStatus(name='status_valid_candidate',
            color='greenyellow',
            text="Valid candidate",
            url="https://people.canonical.com/~ubuntu-archive/proposed-migration/update_excuses.html#\\N")
        self.set_package_status_list([ self._status_ok, self._status_stuck,
            self._status_depwait, self._status_ftbfs, self._status_wrongversion,
            self._status_new, self._status_notfound])

        #Wether to check the update excuses to set some nodes status or not
        self._check_excuses = False

    def set_check_excuses(self,enabled):
        self._check_excuses = True
        self.set_package_status_list([ self._status_ok, self._status_valid_candidate,
            self._status_stuck,
            self._status_depwait, self._status_autopkgtests_running,
            self._status_autopkgtests_failed, self._status_ftbfs,
            self._status_wrongversion,
            self._status_new, self._status_notfound])

    def set_excuses_url(self,url):
        self._update_excuses_url = url

    def set_package_status_from_lp_sources_and_builds(self,all_sources,all_builds,lpseries):
        #For each package check its state and print the proper node info line
        #Parse excuses
        if self._check_excuses:
            excuses_parser = ExcusesHTMLParser()
            excuses_parser.parse_excuses_url(self._update_excuses_url)
            excuses_info_map = excuses_parser.get_package_info()
        #Check all packages
        for package in all_sources:
            sources = all_sources[package]
            if len(sources) == 0:
                #This package is not on Proposed nor Release
                package_uploads_new = lpseries.getPackageUploads(name=package,exact_match=True,status='New')
                if len(package_uploads_new) != 0:
                    self.set_package_status(package,self._status_new)
                else:
                    self.set_package_status(package,self._status_notfound)
            else:
                #Check if the package is at the correct version
                if package not in self._different_versions:
                    actual_version = sources[0].source_package_version.split(':')[-1]
                    expected_version = self._expected_version
                    if not actual_version.startswith(expected_version):
                        #This package has a wrong version
                        self.set_package_status(package,self._status_wrongversion)
                        #Set the package node url in the graph
                        url = sources[0].self_link
                        url = re.sub(r"api\.launchpad\.net/[^/]*","launchpad.net",url) + "/+listing-archive-extra"
                        self.set_package_url(package,url)
                        continue
                #This package is ok, set it as such and continue
                if sources[0].pocket == "Release":
                    self.set_package_status(package,self._status_ok)
                    continue
                #This package is still stuck @ -proposed
                builds = all_builds[package]
                its_ftbfs = False
                its_depwait = False
                for build in builds:
                    #Ignore this build if it's not in the arch list; None means 'consider all'
                    if self._arch_list != None:
                        if build.arch_tag not in self._arch_list:
                            continue
                    #Check build state
                    if build.buildstate == "Dependency wait":
                        its_depwait = True
                    elif build.buildstate == "Failed to build":
                        its_ftbfs = True
                if its_ftbfs:
                    self.set_package_status(package,self._status_ftbfs)
                elif its_depwait:
                    self.set_package_status(package,self._status_depwait)
                else:
                    #Check if the package is stuck in the new queue
                    package_uploads_new = lpseries.getPackageUploads(name=package,exact_match=True,status='New')
                    if len(package_uploads_new) != 0:
                        self.set_package_status(package,self._status_new)
                    else:
                        if not self._check_excuses:
                            self.set_package_status(package,self._status_stuck)
                        else:
                            #Check the excuses status
                            if not its_ftbfs and not its_depwait and package in excuses_info_map:
                                excuses_package_info = excuses_info_map[package]
                                if excuses_package_info.status == "Valid candidate":
                                    self.set_package_status(package,self._status_valid_candidate)
                                else:
                                    found_autopkgtest_failing = False
                                    found_autopkgtest_running = False
                                    for autopkgtest_package in excuses_package_info.autopkgtests:
                                        autopkgtest_info_map = excuses_package_info.autopkgtests[autopkgtest_package]
                                        for autopkgtest_arch in autopkgtest_info_map:
                                            autopkgtest_status = autopkgtest_info_map[autopkgtest_arch]
                                            if autopkgtest_status == "Regression":
                                                found_autopkgtest_failing = True
                                                break
                                            elif autopkgtest_status == "Test in progress":
                                                found_autopkgtest_running = True
                                        if autopkgtest_status == "Regression":
                                            break
                                    if found_autopkgtest_failing:
                                        self.set_package_status(package,self._status_autopkgtests_failed)
                                    elif found_autopkgtest_running:
                                        self.set_package_status(package,self._status_autopkgtests_running)
                                    else:
                                        self.set_package_status(package,self._status_stuck)
                            else:
                                self.set_package_status(package,self._status_stuck)
